GIT_IGNORE_CONTENT = """
# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
*.egg-info/
.installed.cfg
*.egg

# PyInstaller
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.nox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.cover
*.py,cover
.hypothesis/
.pytest_cache/

# Translations
*.mo
*.pot

# Django stuff:
*.log
local_settings.py
db.sqlite3

# virtualenv
venv/
ENV/
env/
env.bak/
venv.bak/

# IDEs
.idea/
.vscode/
"""


import os

def fix_appconfig_name(apps_dir, app_name):
    app_path = os.path.join(apps_dir, app_name)
    apps_py_path = os.path.join(app_path, 'apps.py')

    if not os.path.exists(apps_py_path):
        print(f"❌ {apps_py_path} not found!")
        return

    with open(apps_py_path, 'r') as f:
        lines = f.readlines()

    # Modify the line that sets "name = ..."
    with open(apps_py_path, 'w') as f:
        for line in lines:
            if line.strip().startswith('name ='):
                f.write(f"    name = 'apps.{app_name}'\n")
            else:
                f.write(line)

    print(f"👉 Le ab to app bhi ban gayi'apps.{app_name},'")
    print(f"👉 To register your app in CUSTOM_APPS use app as prefix like'apps.{app_name},'")


DOCKER_CONTENT = """
# Use official Python image
FROM python:3.11-slim

# Set environment variables
ENV PYTHONDONTWRITEBYTECODE 1
ENV PYTHONUNBUFFERED 1

# Set work directory inside container
WORKDIR /app

# Copy requirements first for caching
COPY requirements.txt /app/

# Install dependencies
RUN pip install --upgrade pip
RUN pip install -r requirements.txt

# Copy project files
COPY . /app/

# Expose port 8000 (Django default)
EXPOSE 8000

# Run Django development server
CMD ["python", "manage.py", "runserver", "0.0.0.0:8000"]
"""


import sys
from .project_setup import create_project
from .app_setup import create_app
from . import __version__

def main():
    args = sys.argv[1:]

    if not args or args[0] in ('-h', '--help'):
        print("Usage:\n  brickdjango startproject <project_name>\n  brickdjango startapp <app_name>\n  brickdjango --version")
        sys.exit(0)

    if args[0] in ('--version', '-v'):
        print(f"brickdjango version {__version__}")
        sys.exit(0)

    if len(args) != 2:
        print("Error: Invalid arguments.")
        print("Usage:\n  brickdjango startproject <project_name>\n  brickdjango startapp <app_name>")
        sys.exit(1)

    command, name = args

    if command == 'startproject':
        create_project(name)
    elif command == 'startapp':
        create_app(name)
    else:
        print(f"Unknown command: {command}")
        print("Usage:\n  brickdjango startproject <project_name>\n  brickdjango startapp <app_name>")
        sys.exit(1)

if __name__ == "__main__":
    main()



import os
import subprocess
import sys

from django_custom_structure.docker_data import DOCKER_CONTENT
from django_custom_structure.git_data import GIT_IGNORE_CONTENT
from django_custom_structure.setting_data import DEVELOPMENT_SETTING_BLOCK, PRODUCTION_SETTING_BLOCK
from django_custom_structure.base_data import BASE_SETTING


def create_project(project_name):
    base_path = os.path.join(os.getcwd(), project_name)

    if os.path.exists(base_path):
        print(f"❌ Error: A project named '{project_name}' already exists at {base_path}")
        sys.exit(1)

    os.makedirs(base_path, exist_ok=True)

    # Create virtual environment
    venv_path = os.path.join(base_path, 'venv')
    subprocess.run([sys.executable, '-m', 'venv', venv_path], check=True)

    # Determine OS-specific paths and executable names
    if sys.platform == 'win32':
        venv_bin_dir = 'Scripts'
        pip_executable = 'pip.exe'
        django_admin_executable = 'django-admin.exe'
    else:
        venv_bin_dir = 'bin'
        pip_executable = 'pip'
        django_admin_executable = 'django-admin'

    pip_path = os.path.join(venv_path, venv_bin_dir, pip_executable)
    django_admin_path = os.path.join(venv_path, venv_bin_dir, django_admin_executable)

    # Install Django
    subprocess.run([pip_path, 'install', 'django'], check=True)

    # Install django-environ
    subprocess.run([pip_path, 'install', 'django-environ'], check=True)

    # Start Django project
    subprocess.run([
        django_admin_path,
        'startproject', 'config', base_path
    ], check=True)

    # Create folder structure
    os.makedirs(os.path.join(base_path, 'config', 'settings'), exist_ok=True)
    os.makedirs(os.path.join(base_path, 'apps'), exist_ok=True)
    os.makedirs(os.path.join(base_path, 'base'), exist_ok=True)

    # Move settings.py to settings/base.py and replace contents with BASE_SETTING
    old_settings = os.path.join(base_path, 'config', 'settings.py')
    new_base_settings = os.path.join(base_path, 'config', 'settings', 'base.py')

    with open(new_base_settings, 'w') as f:
        f.write(BASE_SETTING)

    # Create dev and prod settings files
    dev_settings = os.path.join(base_path, 'config', 'settings', 'development.py')
    prod_settings = os.path.join(base_path, 'config', 'settings', 'production.py')

    # Init settings package and apps package
    open(os.path.join(base_path, 'config', 'settings', '__init__.py'), 'a').close()
    open(os.path.join(base_path, 'apps', '__init__.py'), 'a').close()
    open(os.path.join(base_path, 'base', '__init__.py'), 'a').close()

    env_path = os.path.join(base_path, 'config', 'settings', '.env')
    open(env_path, 'a').close()

    sec_key = None
    if os.path.exists(old_settings):
        with open(old_settings, 'r') as f:
            lines = f.readlines()
        for line in lines:
            if 'SECRET_KEY' in line:
                sec_key = line.strip()
                break

        if sec_key:
            sec_key_clean = "".join(sec_key.split())
            with open(env_path, 'a') as f:
                f.write(f'{sec_key_clean}\n')
        else:
            pass

    with open(prod_settings, 'w') as f:
        f.write('\n' + PRODUCTION_SETTING_BLOCK + '\n')

    with open(dev_settings, 'w') as f:
        f.write('\n' + DEVELOPMENT_SETTING_BLOCK + '\n')


    manage_py_path = os.path.join(base_path, 'manage.py')

    if os.path.exists(manage_py_path):
        with open(manage_py_path, 'r') as f:
            content = f.read()

        new_content = content.replace(
            "os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'config.settings')",
            "os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'config.settings.development')"
        )

        if os.path.exists(old_settings):
            os.remove(old_settings)

        with open(manage_py_path, 'w') as f:
            f.write(new_content)


    # Write .gitignore
    with open(os.path.join(base_path, '.gitignore'), 'w') as f:
        f.write(GIT_IGNORE_CONTENT.strip())

    # Write Dockerfile
    dockerfile_path = os.path.join(base_path, "Dockerfile")
    with open(dockerfile_path, 'w') as f:
        f.write(DOCKER_CONTENT.strip())

    # Create requirements.txt using pip freeze from the venv
    with open(os.path.join(base_path, 'requirements.txt'), 'w') as req_file:
        subprocess.run([pip_path, 'freeze'], stdout=req_file, check=True)

    # ✅ Final success message
    print(f"\n🎉 Le bhai ban gaya tera project, '{project_name}' yaha rakha hai {base_path}")
    print("👉 Ab age kya karna hai...:")
    print(f"   1. cd {project_name}")
    print("   2. Activate the virtual environment:")
    print("   3. Run the server:")
    print("      python manage.py runserver")
    print("Le ab moj kar 🎉 🐙")



DEVELOPMENT_SETTING_BLOCK = """from .base import *
import os

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Database
# https://docs.djangoproject.com/en/5.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR , 'db.sqlite3'),
    }
}
"""

PRODUCTION_SETTING_BLOCK = """from .base import *
import os

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

ALLOWED_HOSTS = []

# Database
# https://docs.djangoproject.com/en/5.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR , 'db.sqlite3'),
    }
}
"""


import os
import subprocess
import sys

from .app_config import fix_appconfig_name

def create_app(app_name):
    base_path = os.getcwd()
    manage_path = os.path.join(base_path, 'manage.py')

    # ✅ Ensure this is a Django project directory
    if not os.path.exists(manage_path):
        print("❌ Error: manage.py not found. Are you in the root of a Django project?")
        sys.exit(1)

    # ✅ App will be created inside the "apps/" folder
    apps_dir = os.path.join(base_path, 'apps')
    os.makedirs(apps_dir, exist_ok=True)

    # ✅ Full path where the app should be created
    app_path = os.path.join(apps_dir, app_name)

    if os.path.exists(app_path):
        print(f"⚠️ App '{app_name}' or Dir '{app_name}' already exists.")
        sys.exit(1)

    app_final_path = os.path.join(apps_dir, app_name)
    os.makedirs(app_final_path, exist_ok=True)
    # print(app_final_path)

    # ✅ Call "python manage.py startapp" to use project settings
    subprocess.run([
        sys.executable,
        'manage.py',
        'startapp',
        app_name,
        app_path
    ], check=True)

    print(f"✅ App '{app_name}' created at {app_path}")

    fix_appconfig_name(apps_dir, app_name)



BASE_SETTING = """import environ
import os

env = environ.Env()

environ.Env.read_env()

root = environ.Path(__file__) - 3
apps_root = root.path('apps')
BASE_DIR = root()

# SECURITY WARNING: keep the secret key used in production secret!

SECRET_KEY = env('SECRET_KEY')

# Application definition

DEFAULT_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

THIRDPARTY_APPS = [
    
]

CUSTOM_APPS =[

]

INSTALLED_APPS = DEFAULT_APPS + THIRDPARTY_APPS + CUSTOM_APPS


MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'config.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'config.wsgi.application'

# Password validation
# https://docs.djangoproject.com/en/5.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.2/howto/static-files/

STATIC_URL = 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/5.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
"""


from setuptools import setup, find_packages

setup(
    name='brickdjango',
    version='0.2.1',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'brickdjango = django_custom_structure.__main__:main',
        ]
    },
    author='Amol Balpande',
    author_email='amolbalpande2020@gmail.com',
    description='A clean, modular Django project structure with environment and Docker support.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
    ],
    python_requires='>=3.8',
)
